package cn.com.qmth.stmms.biz.utils;

import java.util.HashSet;
import java.util.Set;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.SetMultimap;

import cn.com.qmth.stmms.biz.exam.model.Marker;

/**
 * Redis zset替代方案，单JVM内部针对已领取未给分任务的管理工具
 * 
 * @author luoshi
 * 
 */
public class CurrentTaskUtil {

    private static SetMultimap<String, TaskEntry> taskMap = HashMultimap.create();

    /**
     * 尝试领取某个任务
     * 
     * @param marker
     * @param libraryId
     * @return
     */
    public static boolean add(Marker marker, int libraryId) {
        String key = getKey(marker);
        TaskEntry obj = new TaskEntry(marker.getId(), libraryId);

        synchronized (CurrentTaskUtil.class) {
            if (taskMap.containsEntry(key, obj)) {
                return false;
            } else {
                taskMap.put(key, obj);
                return true;
            }
        }
    }

    /**
     * 删除某个已完成的任务
     * 
     * @param marker
     * @param libraryId
     */
    public static void remove(Marker marker, int libraryId) {
        TaskEntry obj = new TaskEntry(marker.getId(), libraryId);
        synchronized (CurrentTaskUtil.class) {
            taskMap.remove(getKey(marker), obj);
        }
    }

    /**
     * 某个科目已领取未给分的任务总数
     * 
     * @param examId
     * @param subjectCode
     * @return
     */
    public static int count(int examId, String subjectCode, int number) {
        Set<TaskEntry> set = taskMap.get(getKey(examId, subjectCode, number));
        return set != null ? set.size() : 0;
    }

    /**
     * 某个评卷员已领取未给分的任务总数
     * 
     * @param marker
     * @return
     */
    public static int count(Marker marker) {
        Set<TaskEntry> set = taskMap.get(getKey(marker));
        int count = 0;
        if (set != null) {
            for (TaskEntry obj : set) {
                if (obj.markerId == marker.getId()) {
                    count++;
                }
            }
        }
        return count;
    }

    /**
     * 清除某个科目已领取未给分的任务
     * 
     * @param examId
     * @param subjectCode
     */
    public static void clear(int examId, String subjectCode, int number) {
        synchronized (CurrentTaskUtil.class) {
            taskMap.removeAll(getKey(examId, subjectCode, number));
        }
    }

    /**
     * 清除某个评卷员已领取未给分的任务
     * 
     * @param examId
     * @param subjectCode
     */
    public static void clear(Marker marker) {
        String key = getKey(marker);
        Set<TaskEntry> set = new HashSet<TaskEntry>();
        synchronized (CurrentTaskUtil.class) {
            set.addAll(taskMap.get(key));
            for (TaskEntry obj : set) {
                if (obj.markerId == marker.getId()) {
                    taskMap.remove(key, obj);
                }
            }
        }
    }

    private static String getKey(int examId, String subjectCode, int number) {
        return examId + "_" + subjectCode + "_" + number;
    }

    private static String getKey(Marker marker) {
        return getKey(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
    }
}

class TaskEntry {

    int markerId;

    int libraryId;

    public TaskEntry(int markerId, int libraryId) {
        this.markerId = markerId;
        this.libraryId = libraryId;
    }

    @Override
    public int hashCode() {
        return libraryId;
    }

    @Override
    public String toString() {
        return markerId + "-" + libraryId;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj != null && obj instanceof TaskEntry) {
            TaskEntry mt = (TaskEntry) obj;
            return mt.libraryId == this.libraryId;
        } else {
            return false;
        }
    }
}